from pathlib import Path
from tempfile import TemporaryDirectory
from typing import Iterator

import pytest
from nyl.generator.helmchart import HelmChartGenerator
from nyl.resources import ObjectMetadata
from nyl.resources.helmchart import ChartRef, HelmChart, HelmChartSpec
from nyl.tools.testing import TESTDATA_PATH

CHART_PATH = TESTDATA_PATH / "helm-test-chart"


@pytest.fixture
def generator() -> Iterator[HelmChartGenerator]:
    with TemporaryDirectory() as _tmp:
        tmp = Path(_tmp)
        yield HelmChartGenerator(tmp, tmp, [], tmp, "1.31", set())


def test_helmchart_resource_overrides_labels(generator: HelmChartGenerator) -> None:
    """
    This test verifies that resources generated by a Helm chart component have their labels updated and overwritten
    by labels on the resource that describes the chart instantiation.
    """

    chart_resource = HelmChart(
        metadata=ObjectMetadata(
            name="myapp",
            namespace="mynamespace",
            labels={"app.kubernetes.io/name": "overwritten-value", "team": "testing"},
        ),
        spec=HelmChartSpec(chart=ChartRef(path=str(CHART_PATH))),
    )

    resources = generator.generate(chart_resource)
    assert len(resources) == 1
    assert resources[0]["apiVersion"] == "v1"
    assert resources[0]["kind"] == "ServiceAccount"
    assert resources[0]["metadata"]["labels"] == {
        "app.kubernetes.io/name": "overwritten-value",
        "app.kubernetes.io/version": "1.0.0",
        "team": "testing",
    }
